/*
    FEARLUS model0-6-2: VerticalCylindricalTopology.m
    Copyright (C) 1999-2002  Macaulay Institute
    Copyright (C) 1999-2002  Macaulay Institute
    Copyright (C) 1999-2002  Macaulay Institute

    This file is part of FEARLUS model0-6-2, an agent-based model of land use
    change.

    FEARLUS model0-6-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/


#import "VerticalCylindricalTopology.h"
#import "Coordinate2D.h"

@implementation VerticalCylindricalTopology

+create: aZone setSize: (id <Grid2DSpatial>)size Radius: (unsigned)r {
  VerticalCylindricalTopology *obj;

  obj = [super create: aZone];
  obj->sizeX = [size getX];
  obj->sizeY = [size getY];
  obj->radius = (int)r;
  obj->done = YES;

  obj->returnCoord = [Coordinate2D create: aZone setX: 0 Y: 0];

  return obj;
}

/*

startSearchWithCentre:

Set things up for a new search at the specified centre location.

*/

-(id <Grid2DSpatial>)startSearchWithCentre: (id <Grid2DSpatial>)loc {
  startX = [loc getX];
  startY = [loc getY];

  /* Wrap around in the X dimension */
  if(2 * radius >= sizeX) {	/* The area to search is bigger than the
				   environment -- avoid counting neighbours
				   twice... */
    minX = 0;
    maxX = sizeX - 1;
  }
  else {
    minX = startX - radius;
    maxX = startX + radius;
  }

  /* Bounded in the Y dimension */
  minY = ((startY - radius) < 0) ? 0 : (startY - radius);
  maxY = ((startY + radius) >= sizeY) ? sizeY : (startY + radius);

  returnCoord->x = minX;	/* x and y are public variables in Coordinate2D
				   for the sake of speed. */
  returnCoord->y = minY;

  return returnCoord;
}

-(BOOL)continueSearch {
  return (returnCoord->y <= maxY);
}

-(id <Grid2DSpatial>)nextSearchItem {
  returnCoord->x++;
  if(returnCoord->x > maxX) {
    returnCoord->x = minX;
    returnCoord->y++;
  }

  return returnCoord;
}

-(BOOL)getNeighbourhoodCoordinatesOf: (id <Grid2DSpatial>)loc1
			  relativeTo: (id <Grid2DSpatial>)loc2
				  in: (Coordinate2D *)coord {
  int x1 = [loc1 getX], y1 = [loc1 getY];
  int x2 = [loc2 getX], y2 = [loc2 getY];
  int dx, dy;
  int i;

  for(i = 0; i <= 1; i++) {

    dx = (x1 < x2) ? x2 - x1 : x1 - x2;
    dy = (y1 < y2) ? y2 - y1 : y1 - y2;

    if(dx <= radius && dy <= radius) {
      coord->x = x1;
      coord->y = y1;
      return YES;
    }

    if(dx > radius) x1 = (x1 < x2) ? x1 + sizeX : (x1 == x2) ? x1 : x1 - sizeX;
  }
  return NO;
}

-(void)drop {
  [returnCoord drop];
  [super drop];
}

@end
